/* FileFinderAdaptor.m:
 * A simple adaptor example using the IXFileFinder class from the Indexing Kit.
 * You may freely copy, distribute, and reuse the code in this example.
 * NeXT disclaims any warranty of any kind, expressed or  implied, as to its
 * fitness for any particular use.
 *
 * Written by: David Stutz
 * Revised by: Mai Nguyen, August 1992
 *
 */

#import "FileFinderAdaptor.h"
#import <DBEntity.h>
#import <DBAttribute.h>
#import <dbkit/DBBinder.h>
#import <dbkit/DBValue.h>
#import <indexing/IXFileFinder.h>
#import <indexing/IXFileRecord.h>
#import <indexing/IXPostingList.h>
#import <sys/file.h>
#import <ansi/time.h>
#import <libc.h>

@implementation FileFinderAdaptor : DBBasicAdaptor

- initForDatabase:(DBDatabase*)aDatabase
{
  database = aDatabase;
  [database setName:"FileFinder"];
  onlyEntity = [[DBEntity alloc] initWithName:"Files"
		                 loadableFromDatabase:aDatabase];
  nameProperty = [[DBAttribute alloc] initWithName:"Filename" andType:"*"
		                      forEntity:onlyEntity];
  dateProperty = [[DBAttribute alloc] initWithName:"Date" andType:"*"
		                      forEntity:onlyEntity];
  typeProperty = [[DBAttribute alloc] initWithName:"Type" andType:"*"
		                      forEntity:onlyEntity];
	/* For simplicity, data is set to READ-ONLY */
  [nameProperty setReadOnly:YES];
  [dateProperty setReadOnly:YES];
  [typeProperty setReadOnly:YES];
  return self;
}

/* This method is usually called by the DBDatabase object */
- (BOOL) connectUsingString:(const unsigned char*)aString
{
  char buf[300];
		/* Default behavior: Connect to an indexed directory */
  if (aString == NULL) {
	aString = NXCopyStringBufferFromZone
	("/NextDeveloper/Examples", [self zone]);
  }

  if (isOpen || access(aString, R_OK)) {
    return NO;
  }

  directory = NXCopyStringBufferFromZone(aString, [self zone]);
  strncpy(buf, directory, 300);
  strncat(buf, "/.index.store", 300);
  finder = [[IXFileFinder alloc] initFromName:"FileFinder" inFile:buf
	                         forWriting:NO atPath:directory];

  isOpen = YES;
  return YES;
}

- (BOOL)disconnectUsingString:(const unsigned char*)aString
{
  if (!isOpen) {
    return NO;
  }
  NXZoneFree([self zone], directory);
  directory = NULL;
  [finder free];
  finder = nil;
  isOpen = NO;
  return YES;
}

- (BOOL)isConnected
{
  return isOpen;
}

- (BOOL)insertData:(DBBinder*)aBinder
{
  return NO;
}
 
/* This method gets called when doing a "fetchAllRecords" at the IB level */  
- (BOOL)selectData:(DBBinder*)aBinder
{
	/* This should return all the files in the given root directory */
  return [self evaluateString:"prefix( FileName quote())" using:aBinder];
}
    
- (BOOL)updateData:(DBBinder*)aBinder
{
  return NO;
}
    
- (BOOL)deleteData:(DBBinder*)aBinder
{
  return NO;
}

- (BOOL)evaluateString:(const unsigned char*)aString using:(DBBinder*)aBinder
{
  postings = [finder performQuery:aString atPath:[finder rootPath]
  				 forSender:nil];
  if ((postingCount = [postings count]) == 0) {
    postings = nil;
    postingCount = 0;
    return NO;
  }
  currentRow = 0;
  return YES;
}

- (BOOL)fetchData:(DBBinder*)aBinder
{
  int i,c;
  time_t theDate;
  id props, posting;
 
  if (!isOpen || postings == nil || currentRow >= postingCount) {
    return NO;
  }

  posting = [postings objectAt:currentRow];
  props = [[List alloc] init];
  [aBinder getProperties:props];
  c = [props count];
  
  for (i = 0; i < c; i++) {
    if ([[props objectAt:i] matchesProperty:nameProperty]) {
      [[aBinder valueForProperty:nameProperty]
                  setStringValue:[posting filename]];
    }
    else if ([[props objectAt:i] matchesProperty:dateProperty]) {
      theDate = (time_t)[posting filedate];
      [[aBinder valueForProperty:dateProperty] 
	  			setStringValue:(char *)(ctime(&theDate))];
    }
    else if ([[props objectAt:i] matchesProperty:typeProperty]) {
      [[aBinder valueForProperty:typeProperty]
                  setStringValue:[posting filetype]];
    }
  }
  currentRow++;
  [props free];
  return YES;
}

- (void)fetchDone:(DBBinder*)aBinder
{
  postings = nil;
}

- (const unsigned char*)connectionName
{
  return "FileFinder";
}

- (const unsigned char*)currentLoginString
{
  return directory;
}

- enumerateEntities:(List*)aList
{
  [aList empty];
  if (isOpen) {
    [aList addObject:onlyEntity];
  }
  return nil;
}

- enumerateProperties:(List*)aList forEntity:(id<DBEntities>)anEntity
{
  [aList empty];
  if (isOpen && [anEntity matchesEntity:onlyEntity]) {
    [aList addObject:dateProperty];
    [aList addObject:nameProperty];
    [aList addObject:typeProperty];
  }
  return nil;
}

@end
